 /**********************************************************************
 *
 * langaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/


#include <ctype.h>
#include <stdio.h>
#include <fstream.h>
#include <unistd.h>
#include "langaction.h"
#include "fileutil.h"
#include "gsdltimes.h"
#include "gsdltools.h"
#include "gsdlunicode.h"
#include "langdb.h"
#include "receptionist.h"
#include "errno.h"
#include <sys/utsname.h>

 
/*====================*
 * GLOBAL DEFINITIONS *
 *====================*/

// text file reading buffer
const int LINESIZE = 1024;


/*====================*
 * CLASS DEFINITIONS  *
 *====================*/

langaction::langaction()
{
  package = "";

  cgiarginfo arg_ainfo;

  arg_ainfo.shortname = "l";
  arg_ainfo.longname = "interface language";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "en";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (&cerr, arg_ainfo);

  arg_ainfo.shortname = "slng";
  arg_ainfo.longname = "translation source language";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "en";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (&cerr, arg_ainfo);

  arg_ainfo.shortname = "tlng";
  arg_ainfo.longname = "translation target language";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "fr";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (&cerr, arg_ainfo);
}



langaction::~langaction () {
  // Nothing to do in destructor
}



// set_navbarmacros sets _navigationbar_ and _httpbrowseXXX_ macros
// reponse contains 1 metadata field (Title)
bool langaction::init (ostream & /*logout*/)
{
  // set up GSDLOS, GSDLHOME and PATH environment variables
  text_t gsdlos, path;
  unsigned int path_separator = ':';
#if defined (__WIN32__)
  gsdlos = "windows";
  path_separator = ';';

  path = filename_cat (gsdlhome, "bin", "windows", "perl", "bin;");

#else
  struct utsname *buf = new struct utsname();
  int i = uname (buf);
  if (i == -1) gsdlos = "linux"; // uname failed
  else gsdlos.setcstr (buf->sysname);
  delete buf;
  lc (gsdlos);
#endif

  pathc = getenv ("PATH");
  path += filename_cat (gsdlhome, "bin", gsdlos);
  path.push_back (path_separator);
  path += filename_cat (gsdlhome, "bin", "script");
  if (pathc != NULL) {
    path.push_back (path_separator);
    path += pathc;
  }
  path = "PATH=" + path;

  gsdlos = "GSDLOS=" + gsdlos;
  text_t setgsdlhome = "GSDLHOME=" + gsdlhome;

  // these will be cleaned up in the destructor
  gsdlosc = gsdlos.getcstr();
  gsdlhomec = setgsdlhome.getcstr();
  pathc = path.getcstr();
  
  putenv (gsdlosc);
  putenv (gsdlhomec);
  putenv (pathc);

  return true;
}



bool langaction::check_cgiargs (cgiargsinfoclass &/*argsinfo*/, cgiargsclass &args,
				recptprotolistclass* /*protos*/, ostream &logout)
{
  // Authenticate the user before allowing modifications
  // args["uan"] = 1;
  // args["ug"] = "langadmin";
  return true;
}



void langaction::get_cgihead_info (cgiargsclass &/*args*/, recptprotolistclass* /*protos*/,
				   response_t &response, text_t &response_data, 
				   ostream &logout)
{
  response = content;
  response_data = "text/html";
}



void langaction::define_internal_macros (displayclass &disp, cgiargsclass &args, 
					 recptprotolistclass *protos, ostream &logout)
{
  text_t scriptdir = filename_cat(gsdlhome, "bin", "script", "translator");
  text_t translatordir = filename_cat(gsdlhome, "translator");

  text_t page = args["p"];
  logout << endl << "Page arg: " << page << endl;
  args["p"] = "translang";

  if (page == "translang") {
    // Run the inittranslator.pl script to generate the translation agency start page
    text_t cmd = "perl " + filename_cat(scriptdir, "inittranslator.pl");
    if (gsdl_system(cmd, true, logout) != 0) {
      logout << "Error: Command " << cmd << " did not execute.\n";
      return;
    }

    define_webpage(disp, translatordir, "picklanguage", false, logout);
    return;
  }

  // Get the source and target languages, and file to translate, from the CGI arguments
  text_t sourceabbr = args["slng"];
  text_t targetabbr = args["tlng"];
  text_t transfile = args["file"];

  // Case fold both language codes, just in case
  for (int i = 0; i < strlen(sourceabbr.getcstr()); i++) {
    sourceabbr[i] = tolower(sourceabbr[i]);
  }
  for (int i = 0; i < strlen(targetabbr.getcstr()); i++) {
    targetabbr[i] = tolower(targetabbr[i]);
  }

  logout << "Source abbr: " << sourceabbr << " Target abbr: " << targetabbr << endl;
  logout << "Trans file: " << transfile << endl;

  // overwrite the global content (which should be overwritten by package)
  // makes it obvious that package macros aren't defined, compared to "oops..."
  disp.setmacro("content", displayclass::defaultpackage, "\\_translang:content\\_");

  // Make sure the source and target languages are different
  if (sourceabbr == targetabbr) {
    logout << "Same language!" << endl;
    define_webpage(disp, translatordir, "picklanguage", true, logout);
    return;
  }

  text_t translation = sourceabbr + "-" + targetabbr;
  text_t translationdir = filename_cat(translatordir, translation, transfile);
  logout << "Translation dir: " << translationdir << endl;

  if (page == "translang_picklanguage") {
    // Run the inittranslation.pl script to make sure everything is ready
    text_t cmd = "perl " + filename_cat(scriptdir, "inittranslation.pl") +
      " " + sourceabbr + " " + targetabbr + " " + transfile;
    if (gsdl_system(cmd, true, logout) != 0) {
      logout << "Error: Command " << cmd << " did not execute...\n";
      return;
    }

    // Run the writetranslationmedium.pl script to generate the HTML pages
    cmd = "perl " + filename_cat(scriptdir, "writetranslationmedium.pl") +
      " " + sourceabbr + " " + targetabbr + " " + transfile + " " + "html";
    if (gsdl_system(cmd, true, logout) != 0) {
      logout << "Error: Command " << cmd << " did not execute...\n";
      return;
    }

    // Send an email notification to the site maintainer
    text_t mailfilepath = filename_cat(translationdir, "email.txt");
    char *mailfilenamec = mailfilepath.getcstr();
    ofstream mailfile (mailfilenamec);
    if (!mailfile) {
      logout << "Error: Couldn't write file " << mailfilepath << "." << endl;
      return;
    }

    mailfile << "[Translator Event]\n"
	     << "Date: " << get_date (true) << "\n"
	     << "Greenstone Username: " << args["un"] << "\n"
	     << "The " << targetabbr << " language translation was entered.\n";
    mailfile.close();

    const recptconf &rcinfo = recpt->get_configinfo();

    // Use sendmail.pl perl script to send an email notification
    text_t sendmail_cmd = "perl -S sendmail.pl";
    sendmail_cmd += " -to \"" + rcinfo.maintainer + "\"";
    sendmail_cmd += " -from \"greenstone@cs.waikato.ac.nz\"";
    sendmail_cmd += " -smtp \"" + rcinfo.MailServer + "\"";
    sendmail_cmd += " -subject \"Greenstone Translator Event\"";
    sendmail_cmd += " -msgfile \"" + mailfilepath + "\"";

    // if (gsdl_system(sendmail_cmd, true, logout) != 0) {
    // logout << "Error: Command " << sendmail_cmd << " did not execute...\n";
    // return;
    // }

    // Finished with the temporary file
    unlink(mailfilenamec);
    delete mailfilenamec;

    define_webpage(disp, translationdir, "1", false, logout);
    return;
  }

  // One of the translation pages
  text_tarray splitarray;
  splitchar(page.begin(), page.end(), '_', splitarray);
  text_t subpage = splitarray[0];
  if (splitarray.size() > 1) {
    subpage = splitarray[splitarray.size() - 1];
  }
  logout << "Subpage: " << subpage << endl;

  // Check if the "NEXT" button has been pressed
  if (args[subpage] == "NEXT") {    // !! This is BAD !!
    // Write the submitted translations to a temporary file
    text_t submissionfilepath = filename_cat(translationdir, "submission.tmp");
    ofstream submissionfile(submissionfilepath.getcstr(), ios::out);
    if (!submissionfile) {
      logout << "Error: File " << submissionfilepath << " could not be created." << endl;
      return;
    }

    // Write out the cgiargsclass variable to the submission file
    submissionfile << args;
    submissionfile.close();

    // Make the file world writable
    gsdl_system("chmod a+w " + submissionfilepath, false, logout);

    // Run the submittranslation.pl script to record the translations
    text_t cmd = "perl " + filename_cat(scriptdir, "submittranslations.pl") +
      " " + sourceabbr + " " + targetabbr + " " + transfile + " " + "html";
    if (gsdl_system(cmd, true, logout) != 0) {
      logout << "Error: Command " << cmd << " did not execute...\n";
      return;
    }

    // Move onto the next page
    subpage = atoi(subpage.getcstr()) + 1;
  }

  define_webpage(disp, translationdir, subpage, false, logout);
}



void langaction::define_webpage (displayclass &disp, text_t dir, text_t page, bool error,
				 ostream &logout)
{
  // Open the file to read the webpage from
  text_t webpagefilename = filename_cat(dir, page + ".trans");
  ifstream webpagefile(webpagefilename.getcstr(), ios::in);
  if (!webpagefile) {
    logout << "Error: File " << webpagefilename << " could not be found.\n";
    return;
  }

  // ============================ START HTML CODE ============================
  text_t text = "<html>\n";
  text += "<head>\n";
  text += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n";
  text += "</head>\n";

  text += "<body>\n";
  text += "_texttranshead_\n";

  text += "<input type=hidden name=\"a\" value=\"lang\">\n";
  text += "<input type=hidden name=\"p\" value=\"translang_" + page + "\">\n";
  text += "<input type=hidden name=\"e\" value=\"_decodedcompressedoptions_\">\n";

  // Read in HTML from file
  char line[LINESIZE];
  while (webpagefile.getline(line, LINESIZE-1)) {
    text_t temp = line;
    text += temp + "\n";
  }
  webpagefile.close();

  // Display an error if the source and target languages are the same
  if (page == "picklanguage" && error) {
    text += "<center><strong>_textchoosedifferent_</strong></center>";
  }

  text += "</body>\n";
  text += "</html>\n";

  // if (page == "fromsearch") {
  //   text += "<center><strong> _textcurrpage_ 1 _textof_ 1</strong><br>\n";
  // }

  // ============================ END HTML CODE ============================

  // The following macros are defined in translang.dm
  disp.setmacro("formcontent", "translang", text);
  disp.setmacro("formaction", "translang", "_gwcgi_");

  // Change the navigation bar to include a search button
  text_t navigationbar = "<!-- Navigation Bar -->\n";
  navigationbar += "<table width=\"100%\" cellspacing=0 cellpadding=0>";
  navigationbar += "<tr valign=top>\n";
  navigationbar += "<td rowspan=2 align=left>_imagecollection_</td>";
  navigationbar += "<td align=right>_javalinks_</td>";

  // if (page != "picklanguage") {
  // navigationbar += "<td>_imageserch_</td>";
  // }

  navigationbar += "</tr></table>\n</nobr>\n";
  navigationbar += "<!-- End of Navigation Bar -->\n";
  disp.setmacro ("navigationbar", "translang", navigationbar);
}



bool langaction::do_action (cgiargsclass &args, recptprotolistclass * /*protos*/, 
			    browsermapclass * /*browsers*/, displayclass &disp, 
			    outconvertclass &outconvert, ostream &textout, 
			    ostream &logout) {
  text_t &arg_p = args["p"];

  outconvertclass text2utf8;
  textout << text2utf8 << disp << ("_" + arg_p + ":header_\n")
	  << ("_" + arg_p + ":content_\n")
	  << ("_" + arg_p + ":footer_\n");

  return true;
}




  //if are entering page from macro search
  /* if (args["macroname"] != "") {
    text_t macroname = args["macroname"];

    // !! TO DO !!
    cmd = "perl " + filename_cat(gsdlhome, "bin", "script", "fromsearch.pl") + " " + sourceabbr + " " + targetabbr + " " + macroname;
    logout << "COMMAND: " << cmd <<endl;

    if ((gsdl_system(cmd, true, logout)) != 0) {
      logout << "Process " << cmd << " did not execute.../;-D\n";
      return;
    }

    define_webpage(disp, sourcelang, targetlang, trans_dir, page, -1, false, logout);
    return;
  }

  if (page == "fromsearch")
  page = "thankyou"; */
